
package Java2OWL;
import TestPackage.TestMapper;
import org.semanticweb.owlapi.expression.ParserException;
import org.semanticweb.owlapi.model.OWLNamedIndividual;
import java.util.Set;
import org.semanticweb.owlapi.model.OWLOntology;
import org.semanticweb.owlapi.util.AutoIRIMapper;
import java.io.File;
import java.util.*;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;

/** This class is for testing the accessor groups of type Map.
 *
 * @author ohlbach
 * <p>Execute TestMapper first!
 */
public class MapperTest {
   static J2OManager manager;
    static J2OOntologyManager omg;
    static J2OClassManager clm;
    static J2OIndividualManager img;
    static TestMapper mapper1,mapper2,mapper3;
    
    static Locale loc_de = new Locale("de");
    static Locale loc_en = new Locale("en");  
    static Locale loc_fr = new Locale("fr");
    static IndividualWrapper mapperWr1,mapperWr2,mapperWr3 = null;
    static OWLNamedIndividual mapind1,mapind2,mapind3 = null;
    static ArrayList<IndividualWrapper> iwr = null;
    
    public MapperTest() {}
     
    @BeforeClass
    public static void setUpClass() throws Exception {
        manager = new J2OManager("TestManager",J2OManager.Status.RUN,true);
        manager.getLogger().activateInfo(false);
   
        AutoIRIMapper  iriMapper = new AutoIRIMapper( new File("test/TestPackage"),true);
        omg = manager.getOntologyManager();
        omg.addIRIMapper(iriMapper);
        
        clm = manager.getClassManager();
        String name = "test/TestPackage/TestMapper.owl";
        OWLOntology extOntology = omg.loadExtensionOntology(name);
        omg.setReasoner("Pellet", true, false, 10000);
        
        J2OCompiler comp = manager.getCompiler();
        comp.class2OWL(TestMapper.class);
        comp.finishTranslation();  
         
        img = manager.getIndvidualManager();
        
       
        clm.activateSynchronisation(TestMapper.class);
        mapper1 = new TestMapper("MAP1");
        img.object2OWL(mapper1);
        mapperWr1 = img.getIWR(mapper1);
        mapind1 = mapperWr1.getIndividual();
        
        mapper2 = new TestMapper("MAP2");
        img.object2OWL(mapper2);
        mapperWr2 = img.getIWR(mapper2);
        mapind2 = mapperWr2.getIndividual();
         
        mapper3 = new TestMapper("MAP3");
        img.object2OWL(mapper3);
        mapperWr3 = img.getIWR(mapper3);
        mapind3 = mapperWr3.getIndividual();
        
        
    }
    
    
    @AfterClass
    public static void tearDownClass() throws Exception {
    }
    
    @Before
    public void setUp() {
    }
    
    @After
    public void tearDown() {
    }
    
    @Test
    public void testTextAdd() throws ParserException {
        System.out.println("addText");
        mapper1.addText(loc_de, "Dies ist ein deutscher Text");
        mapper1.addText(loc_en, "This is an English Text");
        assertNotNull(mapperWr1);
        iwr = img.getInstances("hasText_de value \"Dies ist ein deutscher Text\"^^string");
        assertEquals(1,iwr.size());
        assertEquals(mapperWr1, iwr.get(0));
        iwr = img.getInstances("hasText_en value \"This is an English Text\"^^string");
        assertEquals(1,iwr.size());
        assertEquals(mapperWr1, iwr.get(0));
        mapper1.clearText();
        assertNull(omg.getFirstOWLDataProperty("hasText_en"));
        mapper1.addTextLogged(loc_de, "Dies ist ein neuer deutscher Text",false);
        iwr = img.getInstances("hasText_de value \"Dies ist ein neuer deutscher Text\"^^string");
        assertEquals(1,iwr.size());
        assertEquals(mapperWr1, iwr.get(0));
        
        //omg.saveExtensionOntology();
    }
    
    
    @Test
    public void testTextSet() throws ParserException {
        System.out.println("setText");
        mapper1.clearText();
        Map<Locale,String> strings = new HashMap<Locale,String>();
        strings.put(loc_de, "Dies ist ein Text");
        strings.put(loc_en, "This is a Text");
        mapper1.setTexts(strings);
        iwr = img.getInstances("hasText_de value \"Dies ist ein Text\"^^string");
        assertEquals(1,iwr.size());
        assertEquals(mapperWr1, iwr.get(0));
        iwr = img.getInstances("hasText_en value \"This is a Text\"^^string");
        assertEquals(1,iwr.size());
        assertEquals(mapperWr1, iwr.get(0));
        
        Map<Locale,String> newstrings = new HashMap<Locale,String>();
        newstrings.put(loc_de, "Dies ist ein neuer Text");
        newstrings.put(loc_en, "This is a new Text");
        mapper1.setTexts(newstrings);
        iwr = img.getInstances("hasText_de value \"Dies ist ein Text\"^^string");
        assertEquals(0,iwr.size());
        iwr = img.getInstances("hasText_en value \"This is a Text\"^^string");
        assertEquals(0,iwr.size());
        iwr = img.getInstances("hasText_de value \"Dies ist ein neuer Text\"^^string");
        assertEquals(1,iwr.size());
        assertEquals(mapperWr1, iwr.get(0));
        iwr = img.getInstances("hasText_en value \"This is a new Text\"^^string");
        assertEquals(1,iwr.size());
        assertEquals(mapperWr1, iwr.get(0));
    }
        
        
    @Test
    public void testRemoveSet() throws ParserException {
        System.out.println("removeText");
        mapper1.clearText();
        Map<Locale,String> strings = new HashMap<Locale,String>();
        strings.put(loc_de, "Dies ist ein Text");
        strings.put(loc_en, "This is a Text");
        mapper1.setTexts(strings); 
        mapper1.removeText(loc_de); 
        assertNull(omg.getFirstOWLDataProperty("hasText_de"));
        iwr = img.getInstances("hasText_en value \"This is a Text\"^^string");
        assertEquals(1,iwr.size());
        assertEquals(mapperWr1, iwr.get(0));
        
        mapper1.removeTextLogged(loc_en,false); 
        assertNull(omg.getFirstOWLDataProperty("hasText_en"));
    }
    
    @Test
    public void testFriendSet() throws ParserException {
        System.out.println("setFriends");
        HashMap<String,Set<TestMapper>> friendsmap = new HashMap<String,Set<TestMapper>>();
        Set<TestMapper> friends = new HashSet<TestMapper>();
        friends.add(mapper2);
        friends.add(mapper3);
        friendsmap.put("goodFriends",friends);
        mapper1.setFriends(friendsmap);
        iwr = img.getInstances("hasFriends_goodFriends some {Mapper_1}");
        assertEquals(1,iwr.size());
        assertEquals(mapperWr1, iwr.get(0));
        iwr = img.getInstances("hasFriends_goodFriends some {Mapper_0}");
        assertEquals(0,iwr.size());
        
        HashMap<String,Set<TestMapper>> oldfriendsmap = new HashMap<String,Set<TestMapper>>();
        Set<TestMapper> oldfriends = new HashSet<TestMapper>();
        oldfriends.add(mapper1);
        oldfriendsmap.put("oldFriends",oldfriends);
        mapper1.setFriends(oldfriendsmap);
        assertNull(omg.getFirstOWLDataProperty("hasFriends_goodFriends"));
      
        
        iwr = img.getInstances("hasFriends_oldFriends some {Mapper_0}");
        assertEquals(1,iwr.size());
        assertEquals(mapperWr1, iwr.get(0));
        
        mapper1.clearFriends(); 
        assertNull(omg.getFirstOWLDataProperty("hasFriends_oldFriends"));
        
        
        //omg.saveExtensionOntology();
     }
    
    @Test
    public void testFriendAdd() throws ParserException {
        System.out.println("addFriends");
        mapper1.clearFriends();
        mapper1.addFriend("goodFriend",mapper2);
        mapper1.addFriend("goodFriend",mapper3);
        iwr = img.getInstances("hasFriends_goodFriend some {Mapper_1}");
        assertEquals(1,iwr.size());
        assertEquals(mapperWr1, iwr.get(0));
        iwr = img.getInstances("hasFriends_goodFriend some Mapper");
        assertEquals(1,iwr.size());
        assertEquals(mapperWr1, iwr.get(0));
        
        Set<TestMapper> friends = new HashSet<TestMapper>();
        friends.add(mapper1);
        mapper1.addFriends("ownFriend",friends);
        
        iwr = img.getInstances("hasFriends_ownFriend some {Mapper_0}");
        assertEquals(1,iwr.size());
        assertEquals(mapperWr1, iwr.get(0));
    }
    
    @Test
    public void testFriendRemove() throws ParserException {
        System.out.println("removeFriends");
        
        mapper1.clearFriends();
        mapper1.addFriend("goodFriend",mapper2);
        mapper1.addFriend("goodFriend",mapper3);
        iwr = img.getInstances("hasFriends_goodFriend some {Mapper_1}");
        assertEquals(1,iwr.size());
        
        mapper1.removeFriend("goodFriend",mapper2);
        iwr = img.getInstances("hasFriends_goodFriend some {Mapper_1}");
        assertEquals(0,iwr.size());
        
        mapper1.removeFriends("goodFriend");
        assertNull(omg.getFirstOWLDataProperty("hasFriends_goodFriend"));
        
    }
    
    @Test
    public void testOwnerSet() throws ParserException {
        System.out.println("setOwner");
        HashMap<String,String[]> ownermap = new HashMap<String,String[]>();
        String[] owners = new String[]{"Miller","Jones"};
        ownermap.put("regularOwners",owners);
        mapper1.setOwners(ownermap);
        iwr = img.getInstances("hasOwner_regularOwners some {\"Jones\"^^string}");
        assertEquals(1,iwr.size());
        assertEquals(mapperWr1, iwr.get(0));
        iwr = img.getInstances("hasOwner_regularOwners some {\"Miller\"^^string}");
        assertEquals(1,iwr.size());
        assertEquals(mapperWr1, iwr.get(0));
        
        owners = new String[]{"Bush","Obama"};
        ownermap.put("secretOwners",owners);
        mapper1.setOwners(ownermap);
        iwr = img.getInstances("hasOwner_secretOwners some {\"Obama\"^^string}");
        assertEquals(1,iwr.size());
        assertEquals(mapperWr1, iwr.get(0));
        iwr = img.getInstances("hasOwner_regularOwners some {\"Miller\"^^string}");
        assertEquals(1,iwr.size());
        mapper1.clearOwners();
    }
    
    @Test
    public void testOwnerAdd() throws ParserException {
        mapper1.clearOwners();
        System.out.println("setOwner");
        mapper1.addOwner("secretOwners","Charles");
        iwr = img.getInstances("hasOwner_secretOwners some {\"Charles\"^^string}");
        assertEquals(1,iwr.size());
        assertEquals(mapperWr1, iwr.get(0));
        
        String[] owners = new String[]{"Tom","Tim"};
        mapper1.addOwners("secretOwners",owners);
        iwr = img.getInstances("hasOwner_secretOwners some {\"Charles\"^^string}");
        assertEquals(0,iwr.size());
        iwr = img.getInstances("hasOwner_secretOwners some {\"Tom\"^^string}");
        assertEquals(1,iwr.size());
        assertEquals(mapperWr1, iwr.get(0));}
    
    @Test
    public void testOwnerRemove() throws ParserException {
        mapper1.clearOwners();
        System.out.println("removeOwner");
        mapper1.addOwner("secretOwners","Charles");
        mapper1.addOwner("secretOwners","Jaque");
        iwr = img.getInstances("hasOwner_secretOwners some {\"Charles\"^^string}");
        assertEquals(1,iwr.size());
        mapper1.removeOwner("secretOwners","Charles");
        iwr = img.getInstances("hasOwner_secretOwners some {\"Charles\"^^string}");
        assertEquals(0,iwr.size());
        iwr = img.getInstances("hasOwner_secretOwners some {\"Jaque\"^^string}");
        assertEquals(1,iwr.size());
        mapper1.removeOwner("secretOwners","Jaque");
        assertNull(omg.getFirstOWLDataProperty("hasOwner_secretOwners"));
     }
    
    @Test
    public void testSynchronizeData() throws ParserException {
        System.out.println("Synchonize Data"); 
        mapper1.clearText();
        mapperWr1.deactivateSynchronisation(false);
        mapper1.addText(loc_de, "Synchronsationstest");
        mapper1.addText(loc_en, "Synchronzation Test");
        assertNull(omg.getFirstOWLDataProperty("hasText_de"));
        mapperWr1.synchroniseWithOWL();
        assertNotNull(omg.getFirstOWLDataProperty("hasText_de"));
        iwr = img.getInstances("hasText_en value \"Synchronzation Test\"^^string");
        assertEquals(1,iwr.size());
        assertEquals(mapperWr1, iwr.get(0));
        mapperWr1.deactivateSynchronisation(false);
        mapper1.removeText(loc_en);
        mapper1.addText(loc_fr,"Francais");
        mapperWr1.synchroniseWithOWL();
        assertNull(omg.getFirstOWLDataProperty("hasText_en"));
        assertNotNull(omg.getFirstOWLDataProperty("hasText_de"));
        assertNotNull(omg.getFirstOWLDataProperty("hasText_fr"));
        
     }
        
    @Test
    public void testSynchronizeObject() throws ParserException {
        System.out.println("Synchonize Object"); 
        mapper1.clearFriends();
        mapperWr1.deactivateSynchronisation(false);   
        mapper1.addFriend("good",mapper2);
        mapper1.addFriend("good",mapper3);
        mapper1.addFriend("own",mapper1);
        assertNull(omg.getFirstOWLObjectProperty("hasFriends_good"));
        mapperWr1.synchroniseWithOWL();
        assertNotNull(omg.getFirstOWLObjectProperty("hasFriends_good"));
        iwr = img.getInstances("hasFriends_own some Mapper");
        assertEquals(1,iwr.size());
        mapperWr1.deactivateSynchronisation(false);   
        mapper1.removeFriends("good");
        mapper1.addFriend("own",mapper2);
        mapper1.addFriend("foreign",mapper3);
        mapperWr1.synchroniseWithOWL();
        iwr = img.getInstances("hasFriends_own some {Mapper_0}");
        assertEquals(1,iwr.size());
        iwr = img.getInstances("hasFriends_own some {Mapper_1}");
        assertEquals(1,iwr.size()); 
        iwr = img.getInstances("hasFriends_foreign some {Mapper_2}");
        assertEquals(1,iwr.size()); 
        assertNull(omg.getFirstOWLObjectProperty("hasFriends_good"));
    }
    
    @Test
    public void testOWL2Java() throws ParserException {
        System.out.println("OWL2Java"); 
        
        String name = "test/TestPackage/TestMapperExt.owl";
        omg.loadExtensionOntology(name);
        
        Set<OWLNamedIndividual> inds = omg.getInstances("Mapper");
        assertEquals(3,inds.size());
        Iterator<OWLNamedIndividual> it = inds.iterator();
        OWLNamedIndividual NewMapper1 = it.next();
        OWLNamedIndividual NewMapper2 = it.next();
        OWLNamedIndividual NewMapper3 = it.next();
        IndividualWrapper iwr1 = img.individual2Java(NewMapper1);
        assertNotNull(iwr1);
        TestMapper map1 = (TestMapper)(iwr1.getObjects()[0]);
        assertEquals("Hello",map1.getText(loc_en));
        assertEquals("Bonjour",map1.getText(loc_fr));
        Iterator<TestMapper> friends = map1.getFriends("old").iterator();
        assertTrue(friends.hasNext());
        assertEquals("Name:Mapper3",friends.next().getText(loc_en));
        assertEquals("Name:Mapper2",friends.next().getText(loc_en));
        
    }
    
    
}
